#! /usr/bin/python3

# This script was written by Folkert van Heusden <mail@vanheusden.com>.

# pip install pyncraft

from pyncraft.minecraft import Minecraft
import pyncraft.block as block

# apt install python3-pygame

import pygame.midi

# 'address' is the ip-address or hostname of your spigot-
# enabled minecraft server

mc = Minecraft.create(address='192.168.64.28', port=4711)

instrument_map = dict()
instrument_map['DIRT'] = 36  # bass drum
instrument_map['COBBLESTONE'] = 39  # hand clap
instrument_map['PURPLE_GLAZED_TERRACOTTA'] = 46  # open hi-hat

import sys
import time

mc.postToChat("Here's MIDI sequencer")

pygame.midi.init()
player = pygame.midi.Output(0)

def draw_panel(pos, n_tracks, n_ticks):
    for b in range(0, n_ticks + 2):
        mc.setBlock(pos[0] + b, pos[1] + 0, pos[2], 'GOLD_BLOCK')
        # n_ticks + 1: one row for the cursor
        mc.setBlock(pos[0] + b, pos[1] + n_tracks + 1 + 1, pos[2], 'GOLD_BLOCK')

        # glass panel to put 'notes' on
        for glass in range(0, n_tracks):
            mc.setBlock(pos[0] + b, pos[1] + 1 + glass, pos[2] - 1, 'GLASS')

    # position 1 indiciator
    mc.setBlock(pos[0] + 1, pos[1] + 2 + n_tracks + 1, pos[2], 'TORCH')

    for b in range(0, n_tracks + 1 + 2):
        mc.setBlock(pos[0] + 0, pos[1] + b, pos[2] + 0, 'GOLD_BLOCK')
        mc.setBlock(pos[0] + n_ticks + 1, pos[1] + b, pos[2] + 0, 'GOLD_BLOCK')

def sequencer(player, pos, n_tracks, n_ticks, bpm):
    t = 0

    while True:
        mc.setBlock(pos[0] + t + 1, pos[1] + 1 + n_tracks, pos[2] + 0, 'AIR')
        t = (t + 1) % n_ticks
        mc.setBlock(pos[0] + t + 1, pos[1] + 1 + n_tracks, pos[2] + 0, 'GLOWSTONE')

        track = mc.getBlocks(pos[0] + t + 1, pos[1] + 1 + 0, pos[2] + 0, pos[0] + t + 1, pos[1] + 1 + n_tracks - 1, pos[2] + 0)

        for track_index in range(0, n_tracks):
            block_name = track[0][track_index][0]

            if block_name != 'AIR':
                instrument = instrument_map[block_name] - 1 if block_name in instrument_map else hash(block_name) & 127

                print(time.time(), t, block_name, instrument)

                player.note_on(instrument, 127, 9)

        time.sleep(60 / bpm)

# where the sequencer can be found
pos = (-3125, 80, 3125)

n_tracks = 4
n_ticks = 32

bpm = 160

draw_panel(pos, n_tracks, n_ticks)

sequencer(player, pos, n_tracks, n_ticks, bpm)

# it plays percussion instruments when it finds a block of some sort in the
# panel at the position defined above
# use /teleport ... ... ...    (in minecraft) to go to that position.
